# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: api.proto

import sys

_b = sys.version_info[0] < 3 and (lambda x: x) or (lambda x: x.encode("latin1"))
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor.FileDescriptor(
    name="api.proto",
    package="",
    syntax="proto3",
    serialized_options=None,
    serialized_pb=_b(
        '\n\tapi.proto"#\n\x0cHelloRequest\x12\x13\n\x0b\x63lient_info\x18\x01 \x01(\t"Z\n\rHelloResponse\x12\x19\n\x11\x61pi_version_major\x18\x01 \x01(\r\x12\x19\n\x11\x61pi_version_minor\x18\x02 \x01(\r\x12\x13\n\x0bserver_info\x18\x03 \x01(\t""\n\x0e\x43onnectRequest\x12\x10\n\x08password\x18\x01 \x01(\t"+\n\x0f\x43onnectResponse\x12\x18\n\x10invalid_password\x18\x01 \x01(\x08"\x13\n\x11\x44isconnectRequest"\x14\n\x12\x44isconnectResponse"\r\n\x0bPingRequest"\x0e\n\x0cPingResponse"\x13\n\x11\x44\x65viceInfoRequest"\xad\x01\n\x12\x44\x65viceInfoResponse\x12\x15\n\ruses_password\x18\x01 \x01(\x08\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x13\n\x0bmac_address\x18\x03 \x01(\t\x12\x1c\n\x14\x65sphome_core_version\x18\x04 \x01(\t\x12\x18\n\x10\x63ompilation_time\x18\x05 \x01(\t\x12\r\n\x05model\x18\x06 \x01(\t\x12\x16\n\x0ehas_deep_sleep\x18\x07 \x01(\x08"\x15\n\x13ListEntitiesRequest"\x9a\x01\n ListEntitiesBinarySensorResponse\x12\x11\n\tobject_id\x18\x01 \x01(\t\x12\x0b\n\x03key\x18\x02 \x01(\x07\x12\x0c\n\x04name\x18\x03 \x01(\t\x12\x11\n\tunique_id\x18\x04 \x01(\t\x12\x14\n\x0c\x64\x65vice_class\x18\x05 \x01(\t\x12\x1f\n\x17is_status_binary_sensor\x18\x06 \x01(\x08"s\n\x19ListEntitiesCoverResponse\x12\x11\n\tobject_id\x18\x01 \x01(\t\x12\x0b\n\x03key\x18\x02 \x01(\x07\x12\x0c\n\x04name\x18\x03 \x01(\t\x12\x11\n\tunique_id\x18\x04 \x01(\t\x12\x15\n\ris_optimistic\x18\x05 \x01(\x08"\x90\x01\n\x17ListEntitiesFanResponse\x12\x11\n\tobject_id\x18\x01 \x01(\t\x12\x0b\n\x03key\x18\x02 \x01(\x07\x12\x0c\n\x04name\x18\x03 \x01(\t\x12\x11\n\tunique_id\x18\x04 \x01(\t\x12\x1c\n\x14supports_oscillation\x18\x05 \x01(\x08\x12\x16\n\x0esupports_speed\x18\x06 \x01(\x08"\x8a\x02\n\x19ListEntitiesLightResponse\x12\x11\n\tobject_id\x18\x01 \x01(\t\x12\x0b\n\x03key\x18\x02 \x01(\x07\x12\x0c\n\x04name\x18\x03 \x01(\t\x12\x11\n\tunique_id\x18\x04 \x01(\t\x12\x1b\n\x13supports_brightness\x18\x05 \x01(\x08\x12\x14\n\x0csupports_rgb\x18\x06 \x01(\x08\x12\x1c\n\x14supports_white_value\x18\x07 \x01(\x08\x12"\n\x1asupports_color_temperature\x18\x08 \x01(\x08\x12\x12\n\nmin_mireds\x18\t \x01(\x02\x12\x12\n\nmax_mireds\x18\n \x01(\x02\x12\x0f\n\x07\x65\x66\x66\x65\x63ts\x18\x0b \x03(\t"\xa3\x01\n\x1aListEntitiesSensorResponse\x12\x11\n\tobject_id\x18\x01 \x01(\t\x12\x0b\n\x03key\x18\x02 \x01(\x07\x12\x0c\n\x04name\x18\x03 \x01(\t\x12\x11\n\tunique_id\x18\x04 \x01(\t\x12\x0c\n\x04icon\x18\x05 \x01(\t\x12\x1b\n\x13unit_of_measurement\x18\x06 \x01(\t\x12\x19\n\x11\x61\x63\x63uracy_decimals\x18\x07 \x01(\x05"\x7f\n\x1aListEntitiesSwitchResponse\x12\x11\n\tobject_id\x18\x01 \x01(\t\x12\x0b\n\x03key\x18\x02 \x01(\x07\x12\x0c\n\x04name\x18\x03 \x01(\t\x12\x11\n\tunique_id\x18\x04 \x01(\t\x12\x0c\n\x04icon\x18\x05 \x01(\t\x12\x12\n\noptimistic\x18\x06 \x01(\x08"o\n\x1eListEntitiesTextSensorResponse\x12\x11\n\tobject_id\x18\x01 \x01(\t\x12\x0b\n\x03key\x18\x02 \x01(\x07\x12\x0c\n\x04name\x18\x03 \x01(\t\x12\x11\n\tunique_id\x18\x04 \x01(\t\x12\x0c\n\x04icon\x18\x05 \x01(\t"\x1a\n\x18ListEntitiesDoneResponse"\x18\n\x16SubscribeStatesRequest"7\n\x19\x42inarySensorStateResponse\x12\x0b\n\x03key\x18\x01 \x01(\x07\x12\r\n\x05state\x18\x02 \x01(\x08"t\n\x12\x43overStateResponse\x12\x0b\n\x03key\x18\x01 \x01(\x07\x12-\n\x05state\x18\x02 \x01(\x0e\x32\x1e.CoverStateResponse.CoverState""\n\nCoverState\x12\x08\n\x04OPEN\x10\x00\x12\n\n\x06\x43LOSED\x10\x01"]\n\x10\x46\x61nStateResponse\x12\x0b\n\x03key\x18\x01 \x01(\x07\x12\r\n\x05state\x18\x02 \x01(\x08\x12\x13\n\x0boscillating\x18\x03 \x01(\x08\x12\x18\n\x05speed\x18\x04 \x01(\x0e\x32\t.FanSpeed"\xa8\x01\n\x12LightStateResponse\x12\x0b\n\x03key\x18\x01 \x01(\x07\x12\r\n\x05state\x18\x02 \x01(\x08\x12\x12\n\nbrightness\x18\x03 \x01(\x02\x12\x0b\n\x03red\x18\x04 \x01(\x02\x12\r\n\x05green\x18\x05 \x01(\x02\x12\x0c\n\x04\x62lue\x18\x06 \x01(\x02\x12\r\n\x05white\x18\x07 \x01(\x02\x12\x19\n\x11\x63olor_temperature\x18\x08 \x01(\x02\x12\x0e\n\x06\x65\x66\x66\x65\x63t\x18\t \x01(\t"1\n\x13SensorStateResponse\x12\x0b\n\x03key\x18\x01 \x01(\x07\x12\r\n\x05state\x18\x02 \x01(\x02"1\n\x13SwitchStateResponse\x12\x0b\n\x03key\x18\x01 \x01(\x07\x12\r\n\x05state\x18\x02 \x01(\x08"5\n\x17TextSensorStateResponse\x12\x0b\n\x03key\x18\x01 \x01(\x07\x12\r\n\x05state\x18\x02 \x01(\t"\x98\x01\n\x13\x43overCommandRequest\x12\x0b\n\x03key\x18\x01 \x01(\x07\x12\x11\n\thas_state\x18\x02 \x01(\x08\x12\x32\n\x07\x63ommand\x18\x03 \x01(\x0e\x32!.CoverCommandRequest.CoverCommand"-\n\x0c\x43overCommand\x12\x08\n\x04OPEN\x10\x00\x12\t\n\x05\x43LOSE\x10\x01\x12\x08\n\x04STOP\x10\x02"\x9d\x01\n\x11\x46\x61nCommandRequest\x12\x0b\n\x03key\x18\x01 \x01(\x07\x12\x11\n\thas_state\x18\x02 \x01(\x08\x12\r\n\x05state\x18\x03 \x01(\x08\x12\x11\n\thas_speed\x18\x04 \x01(\x08\x12\x18\n\x05speed\x18\x05 \x01(\x0e\x32\t.FanSpeed\x12\x17\n\x0fhas_oscillating\x18\x06 \x01(\x08\x12\x13\n\x0boscillating\x18\x07 \x01(\x08"\x95\x03\n\x13LightCommandRequest\x12\x0b\n\x03key\x18\x01 \x01(\x07\x12\x11\n\thas_state\x18\x02 \x01(\x08\x12\r\n\x05state\x18\x03 \x01(\x08\x12\x16\n\x0ehas_brightness\x18\x04 \x01(\x08\x12\x12\n\nbrightness\x18\x05 \x01(\x02\x12\x0f\n\x07has_rgb\x18\x06 \x01(\x08\x12\x0b\n\x03red\x18\x07 \x01(\x02\x12\r\n\x05green\x18\x08 \x01(\x02\x12\x0c\n\x04\x62lue\x18\t \x01(\x02\x12\x11\n\thas_white\x18\n \x01(\x08\x12\r\n\x05white\x18\x0b \x01(\x02\x12\x1d\n\x15has_color_temperature\x18\x0c \x01(\x08\x12\x19\n\x11\x63olor_temperature\x18\r \x01(\x02\x12\x1d\n\x15has_transition_length\x18\x0e \x01(\x08\x12\x19\n\x11transition_length\x18\x0f \x01(\r\x12\x18\n\x10has_flash_length\x18\x10 \x01(\x08\x12\x14\n\x0c\x66lash_length\x18\x11 \x01(\r\x12\x12\n\nhas_effect\x18\x12 \x01(\x08\x12\x0e\n\x06\x65\x66\x66\x65\x63t\x18\x13 \x01(\t"2\n\x14SwitchCommandRequest\x12\x0b\n\x03key\x18\x01 \x01(\x07\x12\r\n\x05state\x18\x02 \x01(\x08"E\n\x14SubscribeLogsRequest\x12\x18\n\x05level\x18\x01 \x01(\x0e\x32\t.LogLevel\x12\x13\n\x0b\x64ump_config\x18\x02 \x01(\x08"d\n\x15SubscribeLogsResponse\x12\x18\n\x05level\x18\x01 \x01(\x0e\x32\t.LogLevel\x12\x0b\n\x03tag\x18\x02 \x01(\t\x12\x0f\n\x07message\x18\x03 \x01(\t\x12\x13\n\x0bsend_failed\x18\x04 \x01(\x08"\x1e\n\x1cSubscribeServiceCallsRequest"\xdf\x02\n\x13ServiceCallResponse\x12\x0f\n\x07service\x18\x01 \x01(\t\x12,\n\x04\x64\x61ta\x18\x02 \x03(\x0b\x32\x1e.ServiceCallResponse.DataEntry\x12=\n\rdata_template\x18\x03 \x03(\x0b\x32&.ServiceCallResponse.DataTemplateEntry\x12\x36\n\tvariables\x18\x04 \x03(\x0b\x32#.ServiceCallResponse.VariablesEntry\x1a+\n\tDataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\x33\n\x11\x44\x61taTemplateEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\x30\n\x0eVariablesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01"%\n#SubscribeHomeAssistantStatesRequest"8\n#SubscribeHomeAssistantStateResponse\x12\x11\n\tentity_id\x18\x01 \x01(\t">\n\x1aHomeAssistantStateResponse\x12\x11\n\tentity_id\x18\x01 \x01(\t\x12\r\n\x05state\x18\x02 \x01(\t"\x10\n\x0eGetTimeRequest"(\n\x0fGetTimeResponse\x12\x15\n\repoch_seconds\x18\x01 \x01(\x07*)\n\x08\x46\x61nSpeed\x12\x07\n\x03LOW\x10\x00\x12\n\n\x06MEDIUM\x10\x01\x12\x08\n\x04HIGH\x10\x02*]\n\x08LogLevel\x12\x08\n\x04NONE\x10\x00\x12\t\n\x05\x45RROR\x10\x01\x12\x08\n\x04WARN\x10\x02\x12\x08\n\x04INFO\x10\x03\x12\t\n\x05\x44\x45\x42UG\x10\x04\x12\x0b\n\x07VERBOSE\x10\x05\x12\x10\n\x0cVERY_VERBOSE\x10\x06\x62\x06proto3'
    ),
)

_FANSPEED = _descriptor.EnumDescriptor(
    name="FanSpeed",
    full_name="FanSpeed",
    filename=None,
    file=DESCRIPTOR,
    values=[
        _descriptor.EnumValueDescriptor(
            name="LOW", index=0, number=0, serialized_options=None, type=None
        ),
        _descriptor.EnumValueDescriptor(
            name="MEDIUM", index=1, number=1, serialized_options=None, type=None
        ),
        _descriptor.EnumValueDescriptor(
            name="HIGH", index=2, number=2, serialized_options=None, type=None
        ),
    ],
    containing_type=None,
    serialized_options=None,
    serialized_start=3822,
    serialized_end=3863,
)
_sym_db.RegisterEnumDescriptor(_FANSPEED)

FanSpeed = enum_type_wrapper.EnumTypeWrapper(_FANSPEED)
_LOGLEVEL = _descriptor.EnumDescriptor(
    name="LogLevel",
    full_name="LogLevel",
    filename=None,
    file=DESCRIPTOR,
    values=[
        _descriptor.EnumValueDescriptor(
            name="NONE", index=0, number=0, serialized_options=None, type=None
        ),
        _descriptor.EnumValueDescriptor(
            name="ERROR", index=1, number=1, serialized_options=None, type=None
        ),
        _descriptor.EnumValueDescriptor(
            name="WARN", index=2, number=2, serialized_options=None, type=None
        ),
        _descriptor.EnumValueDescriptor(
            name="INFO", index=3, number=3, serialized_options=None, type=None
        ),
        _descriptor.EnumValueDescriptor(
            name="DEBUG", index=4, number=4, serialized_options=None, type=None
        ),
        _descriptor.EnumValueDescriptor(
            name="VERBOSE", index=5, number=5, serialized_options=None, type=None
        ),
        _descriptor.EnumValueDescriptor(
            name="VERY_VERBOSE", index=6, number=6, serialized_options=None, type=None
        ),
    ],
    containing_type=None,
    serialized_options=None,
    serialized_start=3865,
    serialized_end=3958,
)
_sym_db.RegisterEnumDescriptor(_LOGLEVEL)

LogLevel = enum_type_wrapper.EnumTypeWrapper(_LOGLEVEL)
LOW = 0
MEDIUM = 1
HIGH = 2
NONE = 0
ERROR = 1
WARN = 2
INFO = 3
DEBUG = 4
VERBOSE = 5
VERY_VERBOSE = 6


_COVERSTATERESPONSE_COVERSTATE = _descriptor.EnumDescriptor(
    name="CoverState",
    full_name="CoverStateResponse.CoverState",
    filename=None,
    file=DESCRIPTOR,
    values=[
        _descriptor.EnumValueDescriptor(
            name="OPEN", index=0, number=0, serialized_options=None, type=None
        ),
        _descriptor.EnumValueDescriptor(
            name="CLOSED", index=1, number=1, serialized_options=None, type=None
        ),
    ],
    containing_type=None,
    serialized_options=None,
    serialized_start=1808,
    serialized_end=1842,
)
_sym_db.RegisterEnumDescriptor(_COVERSTATERESPONSE_COVERSTATE)

_COVERCOMMANDREQUEST_COVERCOMMAND = _descriptor.EnumDescriptor(
    name="CoverCommand",
    full_name="CoverCommandRequest.CoverCommand",
    filename=None,
    file=DESCRIPTOR,
    values=[
        _descriptor.EnumValueDescriptor(
            name="OPEN", index=0, number=0, serialized_options=None, type=None
        ),
        _descriptor.EnumValueDescriptor(
            name="CLOSE", index=1, number=1, serialized_options=None, type=None
        ),
        _descriptor.EnumValueDescriptor(
            name="STOP", index=2, number=2, serialized_options=None, type=None
        ),
    ],
    containing_type=None,
    serialized_options=None,
    serialized_start=2375,
    serialized_end=2420,
)
_sym_db.RegisterEnumDescriptor(_COVERCOMMANDREQUEST_COVERCOMMAND)


_HELLOREQUEST = _descriptor.Descriptor(
    name="HelloRequest",
    full_name="HelloRequest",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="client_info",
            full_name="HelloRequest.client_info",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=13,
    serialized_end=48,
)


_HELLORESPONSE = _descriptor.Descriptor(
    name="HelloResponse",
    full_name="HelloResponse",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="api_version_major",
            full_name="HelloResponse.api_version_major",
            index=0,
            number=1,
            type=13,
            cpp_type=3,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="api_version_minor",
            full_name="HelloResponse.api_version_minor",
            index=1,
            number=2,
            type=13,
            cpp_type=3,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="server_info",
            full_name="HelloResponse.server_info",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=50,
    serialized_end=140,
)


_CONNECTREQUEST = _descriptor.Descriptor(
    name="ConnectRequest",
    full_name="ConnectRequest",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="password",
            full_name="ConnectRequest.password",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=142,
    serialized_end=176,
)


_CONNECTRESPONSE = _descriptor.Descriptor(
    name="ConnectResponse",
    full_name="ConnectResponse",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="invalid_password",
            full_name="ConnectResponse.invalid_password",
            index=0,
            number=1,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=178,
    serialized_end=221,
)


_DISCONNECTREQUEST = _descriptor.Descriptor(
    name="DisconnectRequest",
    full_name="DisconnectRequest",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=223,
    serialized_end=242,
)


_DISCONNECTRESPONSE = _descriptor.Descriptor(
    name="DisconnectResponse",
    full_name="DisconnectResponse",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=244,
    serialized_end=264,
)


_PINGREQUEST = _descriptor.Descriptor(
    name="PingRequest",
    full_name="PingRequest",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=266,
    serialized_end=279,
)


_PINGRESPONSE = _descriptor.Descriptor(
    name="PingResponse",
    full_name="PingResponse",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=281,
    serialized_end=295,
)


_DEVICEINFOREQUEST = _descriptor.Descriptor(
    name="DeviceInfoRequest",
    full_name="DeviceInfoRequest",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=297,
    serialized_end=316,
)


_DEVICEINFORESPONSE = _descriptor.Descriptor(
    name="DeviceInfoResponse",
    full_name="DeviceInfoResponse",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="uses_password",
            full_name="DeviceInfoResponse.uses_password",
            index=0,
            number=1,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="name",
            full_name="DeviceInfoResponse.name",
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="mac_address",
            full_name="DeviceInfoResponse.mac_address",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="esphome_core_version",
            full_name="DeviceInfoResponse.esphome_core_version",
            index=3,
            number=4,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="compilation_time",
            full_name="DeviceInfoResponse.compilation_time",
            index=4,
            number=5,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="model",
            full_name="DeviceInfoResponse.model",
            index=5,
            number=6,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="has_deep_sleep",
            full_name="DeviceInfoResponse.has_deep_sleep",
            index=6,
            number=7,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=319,
    serialized_end=492,
)


_LISTENTITIESREQUEST = _descriptor.Descriptor(
    name="ListEntitiesRequest",
    full_name="ListEntitiesRequest",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=494,
    serialized_end=515,
)


_LISTENTITIESBINARYSENSORRESPONSE = _descriptor.Descriptor(
    name="ListEntitiesBinarySensorResponse",
    full_name="ListEntitiesBinarySensorResponse",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="object_id",
            full_name="ListEntitiesBinarySensorResponse.object_id",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="key",
            full_name="ListEntitiesBinarySensorResponse.key",
            index=1,
            number=2,
            type=7,
            cpp_type=3,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="name",
            full_name="ListEntitiesBinarySensorResponse.name",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="unique_id",
            full_name="ListEntitiesBinarySensorResponse.unique_id",
            index=3,
            number=4,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="device_class",
            full_name="ListEntitiesBinarySensorResponse.device_class",
            index=4,
            number=5,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="is_status_binary_sensor",
            full_name="ListEntitiesBinarySensorResponse.is_status_binary_sensor",
            index=5,
            number=6,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=518,
    serialized_end=672,
)


_LISTENTITIESCOVERRESPONSE = _descriptor.Descriptor(
    name="ListEntitiesCoverResponse",
    full_name="ListEntitiesCoverResponse",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="object_id",
            full_name="ListEntitiesCoverResponse.object_id",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="key",
            full_name="ListEntitiesCoverResponse.key",
            index=1,
            number=2,
            type=7,
            cpp_type=3,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="name",
            full_name="ListEntitiesCoverResponse.name",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="unique_id",
            full_name="ListEntitiesCoverResponse.unique_id",
            index=3,
            number=4,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="is_optimistic",
            full_name="ListEntitiesCoverResponse.is_optimistic",
            index=4,
            number=5,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=674,
    serialized_end=789,
)


_LISTENTITIESFANRESPONSE = _descriptor.Descriptor(
    name="ListEntitiesFanResponse",
    full_name="ListEntitiesFanResponse",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="object_id",
            full_name="ListEntitiesFanResponse.object_id",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="key",
            full_name="ListEntitiesFanResponse.key",
            index=1,
            number=2,
            type=7,
            cpp_type=3,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="name",
            full_name="ListEntitiesFanResponse.name",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="unique_id",
            full_name="ListEntitiesFanResponse.unique_id",
            index=3,
            number=4,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="supports_oscillation",
            full_name="ListEntitiesFanResponse.supports_oscillation",
            index=4,
            number=5,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="supports_speed",
            full_name="ListEntitiesFanResponse.supports_speed",
            index=5,
            number=6,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=792,
    serialized_end=936,
)


_LISTENTITIESLIGHTRESPONSE = _descriptor.Descriptor(
    name="ListEntitiesLightResponse",
    full_name="ListEntitiesLightResponse",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="object_id",
            full_name="ListEntitiesLightResponse.object_id",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="key",
            full_name="ListEntitiesLightResponse.key",
            index=1,
            number=2,
            type=7,
            cpp_type=3,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="name",
            full_name="ListEntitiesLightResponse.name",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="unique_id",
            full_name="ListEntitiesLightResponse.unique_id",
            index=3,
            number=4,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="supports_brightness",
            full_name="ListEntitiesLightResponse.supports_brightness",
            index=4,
            number=5,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="supports_rgb",
            full_name="ListEntitiesLightResponse.supports_rgb",
            index=5,
            number=6,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="supports_white_value",
            full_name="ListEntitiesLightResponse.supports_white_value",
            index=6,
            number=7,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="supports_color_temperature",
            full_name="ListEntitiesLightResponse.supports_color_temperature",
            index=7,
            number=8,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="min_mireds",
            full_name="ListEntitiesLightResponse.min_mireds",
            index=8,
            number=9,
            type=2,
            cpp_type=6,
            label=1,
            has_default_value=False,
            default_value=float(0),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="max_mireds",
            full_name="ListEntitiesLightResponse.max_mireds",
            index=9,
            number=10,
            type=2,
            cpp_type=6,
            label=1,
            has_default_value=False,
            default_value=float(0),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="effects",
            full_name="ListEntitiesLightResponse.effects",
            index=10,
            number=11,
            type=9,
            cpp_type=9,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=939,
    serialized_end=1205,
)


_LISTENTITIESSENSORRESPONSE = _descriptor.Descriptor(
    name="ListEntitiesSensorResponse",
    full_name="ListEntitiesSensorResponse",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="object_id",
            full_name="ListEntitiesSensorResponse.object_id",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="key",
            full_name="ListEntitiesSensorResponse.key",
            index=1,
            number=2,
            type=7,
            cpp_type=3,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="name",
            full_name="ListEntitiesSensorResponse.name",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="unique_id",
            full_name="ListEntitiesSensorResponse.unique_id",
            index=3,
            number=4,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="icon",
            full_name="ListEntitiesSensorResponse.icon",
            index=4,
            number=5,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="unit_of_measurement",
            full_name="ListEntitiesSensorResponse.unit_of_measurement",
            index=5,
            number=6,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="accuracy_decimals",
            full_name="ListEntitiesSensorResponse.accuracy_decimals",
            index=6,
            number=7,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=1208,
    serialized_end=1371,
)


_LISTENTITIESSWITCHRESPONSE = _descriptor.Descriptor(
    name="ListEntitiesSwitchResponse",
    full_name="ListEntitiesSwitchResponse",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="object_id",
            full_name="ListEntitiesSwitchResponse.object_id",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="key",
            full_name="ListEntitiesSwitchResponse.key",
            index=1,
            number=2,
            type=7,
            cpp_type=3,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="name",
            full_name="ListEntitiesSwitchResponse.name",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="unique_id",
            full_name="ListEntitiesSwitchResponse.unique_id",
            index=3,
            number=4,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="icon",
            full_name="ListEntitiesSwitchResponse.icon",
            index=4,
            number=5,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="optimistic",
            full_name="ListEntitiesSwitchResponse.optimistic",
            index=5,
            number=6,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=1373,
    serialized_end=1500,
)


_LISTENTITIESTEXTSENSORRESPONSE = _descriptor.Descriptor(
    name="ListEntitiesTextSensorResponse",
    full_name="ListEntitiesTextSensorResponse",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="object_id",
            full_name="ListEntitiesTextSensorResponse.object_id",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="key",
            full_name="ListEntitiesTextSensorResponse.key",
            index=1,
            number=2,
            type=7,
            cpp_type=3,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="name",
            full_name="ListEntitiesTextSensorResponse.name",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="unique_id",
            full_name="ListEntitiesTextSensorResponse.unique_id",
            index=3,
            number=4,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="icon",
            full_name="ListEntitiesTextSensorResponse.icon",
            index=4,
            number=5,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=1502,
    serialized_end=1613,
)


_LISTENTITIESDONERESPONSE = _descriptor.Descriptor(
    name="ListEntitiesDoneResponse",
    full_name="ListEntitiesDoneResponse",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=1615,
    serialized_end=1641,
)


_SUBSCRIBESTATESREQUEST = _descriptor.Descriptor(
    name="SubscribeStatesRequest",
    full_name="SubscribeStatesRequest",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=1643,
    serialized_end=1667,
)


_BINARYSENSORSTATERESPONSE = _descriptor.Descriptor(
    name="BinarySensorStateResponse",
    full_name="BinarySensorStateResponse",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="key",
            full_name="BinarySensorStateResponse.key",
            index=0,
            number=1,
            type=7,
            cpp_type=3,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="state",
            full_name="BinarySensorStateResponse.state",
            index=1,
            number=2,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=1669,
    serialized_end=1724,
)


_COVERSTATERESPONSE = _descriptor.Descriptor(
    name="CoverStateResponse",
    full_name="CoverStateResponse",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="key",
            full_name="CoverStateResponse.key",
            index=0,
            number=1,
            type=7,
            cpp_type=3,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="state",
            full_name="CoverStateResponse.state",
            index=1,
            number=2,
            type=14,
            cpp_type=8,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[
        _COVERSTATERESPONSE_COVERSTATE,
    ],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=1726,
    serialized_end=1842,
)


_FANSTATERESPONSE = _descriptor.Descriptor(
    name="FanStateResponse",
    full_name="FanStateResponse",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="key",
            full_name="FanStateResponse.key",
            index=0,
            number=1,
            type=7,
            cpp_type=3,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="state",
            full_name="FanStateResponse.state",
            index=1,
            number=2,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="oscillating",
            full_name="FanStateResponse.oscillating",
            index=2,
            number=3,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="speed",
            full_name="FanStateResponse.speed",
            index=3,
            number=4,
            type=14,
            cpp_type=8,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=1844,
    serialized_end=1937,
)


_LIGHTSTATERESPONSE = _descriptor.Descriptor(
    name="LightStateResponse",
    full_name="LightStateResponse",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="key",
            full_name="LightStateResponse.key",
            index=0,
            number=1,
            type=7,
            cpp_type=3,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="state",
            full_name="LightStateResponse.state",
            index=1,
            number=2,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="brightness",
            full_name="LightStateResponse.brightness",
            index=2,
            number=3,
            type=2,
            cpp_type=6,
            label=1,
            has_default_value=False,
            default_value=float(0),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="red",
            full_name="LightStateResponse.red",
            index=3,
            number=4,
            type=2,
            cpp_type=6,
            label=1,
            has_default_value=False,
            default_value=float(0),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="green",
            full_name="LightStateResponse.green",
            index=4,
            number=5,
            type=2,
            cpp_type=6,
            label=1,
            has_default_value=False,
            default_value=float(0),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="blue",
            full_name="LightStateResponse.blue",
            index=5,
            number=6,
            type=2,
            cpp_type=6,
            label=1,
            has_default_value=False,
            default_value=float(0),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="white",
            full_name="LightStateResponse.white",
            index=6,
            number=7,
            type=2,
            cpp_type=6,
            label=1,
            has_default_value=False,
            default_value=float(0),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="color_temperature",
            full_name="LightStateResponse.color_temperature",
            index=7,
            number=8,
            type=2,
            cpp_type=6,
            label=1,
            has_default_value=False,
            default_value=float(0),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="effect",
            full_name="LightStateResponse.effect",
            index=8,
            number=9,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=1940,
    serialized_end=2108,
)


_SENSORSTATERESPONSE = _descriptor.Descriptor(
    name="SensorStateResponse",
    full_name="SensorStateResponse",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="key",
            full_name="SensorStateResponse.key",
            index=0,
            number=1,
            type=7,
            cpp_type=3,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="state",
            full_name="SensorStateResponse.state",
            index=1,
            number=2,
            type=2,
            cpp_type=6,
            label=1,
            has_default_value=False,
            default_value=float(0),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=2110,
    serialized_end=2159,
)


_SWITCHSTATERESPONSE = _descriptor.Descriptor(
    name="SwitchStateResponse",
    full_name="SwitchStateResponse",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="key",
            full_name="SwitchStateResponse.key",
            index=0,
            number=1,
            type=7,
            cpp_type=3,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="state",
            full_name="SwitchStateResponse.state",
            index=1,
            number=2,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=2161,
    serialized_end=2210,
)


_TEXTSENSORSTATERESPONSE = _descriptor.Descriptor(
    name="TextSensorStateResponse",
    full_name="TextSensorStateResponse",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="key",
            full_name="TextSensorStateResponse.key",
            index=0,
            number=1,
            type=7,
            cpp_type=3,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="state",
            full_name="TextSensorStateResponse.state",
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=2212,
    serialized_end=2265,
)


_COVERCOMMANDREQUEST = _descriptor.Descriptor(
    name="CoverCommandRequest",
    full_name="CoverCommandRequest",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="key",
            full_name="CoverCommandRequest.key",
            index=0,
            number=1,
            type=7,
            cpp_type=3,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="has_state",
            full_name="CoverCommandRequest.has_state",
            index=1,
            number=2,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="command",
            full_name="CoverCommandRequest.command",
            index=2,
            number=3,
            type=14,
            cpp_type=8,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[
        _COVERCOMMANDREQUEST_COVERCOMMAND,
    ],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=2268,
    serialized_end=2420,
)


_FANCOMMANDREQUEST = _descriptor.Descriptor(
    name="FanCommandRequest",
    full_name="FanCommandRequest",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="key",
            full_name="FanCommandRequest.key",
            index=0,
            number=1,
            type=7,
            cpp_type=3,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="has_state",
            full_name="FanCommandRequest.has_state",
            index=1,
            number=2,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="state",
            full_name="FanCommandRequest.state",
            index=2,
            number=3,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="has_speed",
            full_name="FanCommandRequest.has_speed",
            index=3,
            number=4,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="speed",
            full_name="FanCommandRequest.speed",
            index=4,
            number=5,
            type=14,
            cpp_type=8,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="has_oscillating",
            full_name="FanCommandRequest.has_oscillating",
            index=5,
            number=6,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="oscillating",
            full_name="FanCommandRequest.oscillating",
            index=6,
            number=7,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=2423,
    serialized_end=2580,
)


_LIGHTCOMMANDREQUEST = _descriptor.Descriptor(
    name="LightCommandRequest",
    full_name="LightCommandRequest",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="key",
            full_name="LightCommandRequest.key",
            index=0,
            number=1,
            type=7,
            cpp_type=3,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="has_state",
            full_name="LightCommandRequest.has_state",
            index=1,
            number=2,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="state",
            full_name="LightCommandRequest.state",
            index=2,
            number=3,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="has_brightness",
            full_name="LightCommandRequest.has_brightness",
            index=3,
            number=4,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="brightness",
            full_name="LightCommandRequest.brightness",
            index=4,
            number=5,
            type=2,
            cpp_type=6,
            label=1,
            has_default_value=False,
            default_value=float(0),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="has_rgb",
            full_name="LightCommandRequest.has_rgb",
            index=5,
            number=6,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="red",
            full_name="LightCommandRequest.red",
            index=6,
            number=7,
            type=2,
            cpp_type=6,
            label=1,
            has_default_value=False,
            default_value=float(0),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="green",
            full_name="LightCommandRequest.green",
            index=7,
            number=8,
            type=2,
            cpp_type=6,
            label=1,
            has_default_value=False,
            default_value=float(0),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="blue",
            full_name="LightCommandRequest.blue",
            index=8,
            number=9,
            type=2,
            cpp_type=6,
            label=1,
            has_default_value=False,
            default_value=float(0),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="has_white",
            full_name="LightCommandRequest.has_white",
            index=9,
            number=10,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="white",
            full_name="LightCommandRequest.white",
            index=10,
            number=11,
            type=2,
            cpp_type=6,
            label=1,
            has_default_value=False,
            default_value=float(0),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="has_color_temperature",
            full_name="LightCommandRequest.has_color_temperature",
            index=11,
            number=12,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="color_temperature",
            full_name="LightCommandRequest.color_temperature",
            index=12,
            number=13,
            type=2,
            cpp_type=6,
            label=1,
            has_default_value=False,
            default_value=float(0),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="has_transition_length",
            full_name="LightCommandRequest.has_transition_length",
            index=13,
            number=14,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="transition_length",
            full_name="LightCommandRequest.transition_length",
            index=14,
            number=15,
            type=13,
            cpp_type=3,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="has_flash_length",
            full_name="LightCommandRequest.has_flash_length",
            index=15,
            number=16,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="flash_length",
            full_name="LightCommandRequest.flash_length",
            index=16,
            number=17,
            type=13,
            cpp_type=3,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="has_effect",
            full_name="LightCommandRequest.has_effect",
            index=17,
            number=18,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="effect",
            full_name="LightCommandRequest.effect",
            index=18,
            number=19,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=2583,
    serialized_end=2988,
)


_SWITCHCOMMANDREQUEST = _descriptor.Descriptor(
    name="SwitchCommandRequest",
    full_name="SwitchCommandRequest",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="key",
            full_name="SwitchCommandRequest.key",
            index=0,
            number=1,
            type=7,
            cpp_type=3,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="state",
            full_name="SwitchCommandRequest.state",
            index=1,
            number=2,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=2990,
    serialized_end=3040,
)


_SUBSCRIBELOGSREQUEST = _descriptor.Descriptor(
    name="SubscribeLogsRequest",
    full_name="SubscribeLogsRequest",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="level",
            full_name="SubscribeLogsRequest.level",
            index=0,
            number=1,
            type=14,
            cpp_type=8,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="dump_config",
            full_name="SubscribeLogsRequest.dump_config",
            index=1,
            number=2,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=3042,
    serialized_end=3111,
)


_SUBSCRIBELOGSRESPONSE = _descriptor.Descriptor(
    name="SubscribeLogsResponse",
    full_name="SubscribeLogsResponse",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="level",
            full_name="SubscribeLogsResponse.level",
            index=0,
            number=1,
            type=14,
            cpp_type=8,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="tag",
            full_name="SubscribeLogsResponse.tag",
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="message",
            full_name="SubscribeLogsResponse.message",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="send_failed",
            full_name="SubscribeLogsResponse.send_failed",
            index=3,
            number=4,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=3113,
    serialized_end=3213,
)


_SUBSCRIBESERVICECALLSREQUEST = _descriptor.Descriptor(
    name="SubscribeServiceCallsRequest",
    full_name="SubscribeServiceCallsRequest",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=3215,
    serialized_end=3245,
)


_SERVICECALLRESPONSE_DATAENTRY = _descriptor.Descriptor(
    name="DataEntry",
    full_name="ServiceCallResponse.DataEntry",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="key",
            full_name="ServiceCallResponse.DataEntry.key",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="value",
            full_name="ServiceCallResponse.DataEntry.value",
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=_b("8\001"),
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=3453,
    serialized_end=3496,
)

_SERVICECALLRESPONSE_DATATEMPLATEENTRY = _descriptor.Descriptor(
    name="DataTemplateEntry",
    full_name="ServiceCallResponse.DataTemplateEntry",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="key",
            full_name="ServiceCallResponse.DataTemplateEntry.key",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="value",
            full_name="ServiceCallResponse.DataTemplateEntry.value",
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=_b("8\001"),
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=3498,
    serialized_end=3549,
)

_SERVICECALLRESPONSE_VARIABLESENTRY = _descriptor.Descriptor(
    name="VariablesEntry",
    full_name="ServiceCallResponse.VariablesEntry",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="key",
            full_name="ServiceCallResponse.VariablesEntry.key",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="value",
            full_name="ServiceCallResponse.VariablesEntry.value",
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=_b("8\001"),
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=3551,
    serialized_end=3599,
)

_SERVICECALLRESPONSE = _descriptor.Descriptor(
    name="ServiceCallResponse",
    full_name="ServiceCallResponse",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="service",
            full_name="ServiceCallResponse.service",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="data",
            full_name="ServiceCallResponse.data",
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="data_template",
            full_name="ServiceCallResponse.data_template",
            index=2,
            number=3,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="variables",
            full_name="ServiceCallResponse.variables",
            index=3,
            number=4,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[
        _SERVICECALLRESPONSE_DATAENTRY,
        _SERVICECALLRESPONSE_DATATEMPLATEENTRY,
        _SERVICECALLRESPONSE_VARIABLESENTRY,
    ],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=3248,
    serialized_end=3599,
)


_SUBSCRIBEHOMEASSISTANTSTATESREQUEST = _descriptor.Descriptor(
    name="SubscribeHomeAssistantStatesRequest",
    full_name="SubscribeHomeAssistantStatesRequest",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=3601,
    serialized_end=3638,
)


_SUBSCRIBEHOMEASSISTANTSTATERESPONSE = _descriptor.Descriptor(
    name="SubscribeHomeAssistantStateResponse",
    full_name="SubscribeHomeAssistantStateResponse",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="entity_id",
            full_name="SubscribeHomeAssistantStateResponse.entity_id",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=3640,
    serialized_end=3696,
)


_HOMEASSISTANTSTATERESPONSE = _descriptor.Descriptor(
    name="HomeAssistantStateResponse",
    full_name="HomeAssistantStateResponse",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="entity_id",
            full_name="HomeAssistantStateResponse.entity_id",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
        _descriptor.FieldDescriptor(
            name="state",
            full_name="HomeAssistantStateResponse.state",
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=_b("").decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=3698,
    serialized_end=3760,
)


_GETTIMEREQUEST = _descriptor.Descriptor(
    name="GetTimeRequest",
    full_name="GetTimeRequest",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=3762,
    serialized_end=3778,
)


_GETTIMERESPONSE = _descriptor.Descriptor(
    name="GetTimeResponse",
    full_name="GetTimeResponse",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name="epoch_seconds",
            full_name="GetTimeResponse.epoch_seconds",
            index=0,
            number=1,
            type=7,
            cpp_type=3,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=3780,
    serialized_end=3820,
)

_COVERSTATERESPONSE.fields_by_name["state"].enum_type = _COVERSTATERESPONSE_COVERSTATE
_COVERSTATERESPONSE_COVERSTATE.containing_type = _COVERSTATERESPONSE
_FANSTATERESPONSE.fields_by_name["speed"].enum_type = _FANSPEED
_COVERCOMMANDREQUEST.fields_by_name[
    "command"
].enum_type = _COVERCOMMANDREQUEST_COVERCOMMAND
_COVERCOMMANDREQUEST_COVERCOMMAND.containing_type = _COVERCOMMANDREQUEST
_FANCOMMANDREQUEST.fields_by_name["speed"].enum_type = _FANSPEED
_SUBSCRIBELOGSREQUEST.fields_by_name["level"].enum_type = _LOGLEVEL
_SUBSCRIBELOGSRESPONSE.fields_by_name["level"].enum_type = _LOGLEVEL
_SERVICECALLRESPONSE_DATAENTRY.containing_type = _SERVICECALLRESPONSE
_SERVICECALLRESPONSE_DATATEMPLATEENTRY.containing_type = _SERVICECALLRESPONSE
_SERVICECALLRESPONSE_VARIABLESENTRY.containing_type = _SERVICECALLRESPONSE
_SERVICECALLRESPONSE.fields_by_name[
    "data"
].message_type = _SERVICECALLRESPONSE_DATAENTRY
_SERVICECALLRESPONSE.fields_by_name[
    "data_template"
].message_type = _SERVICECALLRESPONSE_DATATEMPLATEENTRY
_SERVICECALLRESPONSE.fields_by_name[
    "variables"
].message_type = _SERVICECALLRESPONSE_VARIABLESENTRY
DESCRIPTOR.message_types_by_name["HelloRequest"] = _HELLOREQUEST
DESCRIPTOR.message_types_by_name["HelloResponse"] = _HELLORESPONSE
DESCRIPTOR.message_types_by_name["ConnectRequest"] = _CONNECTREQUEST
DESCRIPTOR.message_types_by_name["ConnectResponse"] = _CONNECTRESPONSE
DESCRIPTOR.message_types_by_name["DisconnectRequest"] = _DISCONNECTREQUEST
DESCRIPTOR.message_types_by_name["DisconnectResponse"] = _DISCONNECTRESPONSE
DESCRIPTOR.message_types_by_name["PingRequest"] = _PINGREQUEST
DESCRIPTOR.message_types_by_name["PingResponse"] = _PINGRESPONSE
DESCRIPTOR.message_types_by_name["DeviceInfoRequest"] = _DEVICEINFOREQUEST
DESCRIPTOR.message_types_by_name["DeviceInfoResponse"] = _DEVICEINFORESPONSE
DESCRIPTOR.message_types_by_name["ListEntitiesRequest"] = _LISTENTITIESREQUEST
DESCRIPTOR.message_types_by_name[
    "ListEntitiesBinarySensorResponse"
] = _LISTENTITIESBINARYSENSORRESPONSE
DESCRIPTOR.message_types_by_name[
    "ListEntitiesCoverResponse"
] = _LISTENTITIESCOVERRESPONSE
DESCRIPTOR.message_types_by_name["ListEntitiesFanResponse"] = _LISTENTITIESFANRESPONSE
DESCRIPTOR.message_types_by_name[
    "ListEntitiesLightResponse"
] = _LISTENTITIESLIGHTRESPONSE
DESCRIPTOR.message_types_by_name[
    "ListEntitiesSensorResponse"
] = _LISTENTITIESSENSORRESPONSE
DESCRIPTOR.message_types_by_name[
    "ListEntitiesSwitchResponse"
] = _LISTENTITIESSWITCHRESPONSE
DESCRIPTOR.message_types_by_name[
    "ListEntitiesTextSensorResponse"
] = _LISTENTITIESTEXTSENSORRESPONSE
DESCRIPTOR.message_types_by_name["ListEntitiesDoneResponse"] = _LISTENTITIESDONERESPONSE
DESCRIPTOR.message_types_by_name["SubscribeStatesRequest"] = _SUBSCRIBESTATESREQUEST
DESCRIPTOR.message_types_by_name[
    "BinarySensorStateResponse"
] = _BINARYSENSORSTATERESPONSE
DESCRIPTOR.message_types_by_name["CoverStateResponse"] = _COVERSTATERESPONSE
DESCRIPTOR.message_types_by_name["FanStateResponse"] = _FANSTATERESPONSE
DESCRIPTOR.message_types_by_name["LightStateResponse"] = _LIGHTSTATERESPONSE
DESCRIPTOR.message_types_by_name["SensorStateResponse"] = _SENSORSTATERESPONSE
DESCRIPTOR.message_types_by_name["SwitchStateResponse"] = _SWITCHSTATERESPONSE
DESCRIPTOR.message_types_by_name["TextSensorStateResponse"] = _TEXTSENSORSTATERESPONSE
DESCRIPTOR.message_types_by_name["CoverCommandRequest"] = _COVERCOMMANDREQUEST
DESCRIPTOR.message_types_by_name["FanCommandRequest"] = _FANCOMMANDREQUEST
DESCRIPTOR.message_types_by_name["LightCommandRequest"] = _LIGHTCOMMANDREQUEST
DESCRIPTOR.message_types_by_name["SwitchCommandRequest"] = _SWITCHCOMMANDREQUEST
DESCRIPTOR.message_types_by_name["SubscribeLogsRequest"] = _SUBSCRIBELOGSREQUEST
DESCRIPTOR.message_types_by_name["SubscribeLogsResponse"] = _SUBSCRIBELOGSRESPONSE
DESCRIPTOR.message_types_by_name[
    "SubscribeServiceCallsRequest"
] = _SUBSCRIBESERVICECALLSREQUEST
DESCRIPTOR.message_types_by_name["ServiceCallResponse"] = _SERVICECALLRESPONSE
DESCRIPTOR.message_types_by_name[
    "SubscribeHomeAssistantStatesRequest"
] = _SUBSCRIBEHOMEASSISTANTSTATESREQUEST
DESCRIPTOR.message_types_by_name[
    "SubscribeHomeAssistantStateResponse"
] = _SUBSCRIBEHOMEASSISTANTSTATERESPONSE
DESCRIPTOR.message_types_by_name[
    "HomeAssistantStateResponse"
] = _HOMEASSISTANTSTATERESPONSE
DESCRIPTOR.message_types_by_name["GetTimeRequest"] = _GETTIMEREQUEST
DESCRIPTOR.message_types_by_name["GetTimeResponse"] = _GETTIMERESPONSE
DESCRIPTOR.enum_types_by_name["FanSpeed"] = _FANSPEED
DESCRIPTOR.enum_types_by_name["LogLevel"] = _LOGLEVEL
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

HelloRequest = _reflection.GeneratedProtocolMessageType(
    "HelloRequest",
    (_message.Message,),
    dict(
        DESCRIPTOR=_HELLOREQUEST,
        __module__="api_pb2"
        # @@protoc_insertion_point(class_scope:HelloRequest)
    ),
)
_sym_db.RegisterMessage(HelloRequest)

HelloResponse = _reflection.GeneratedProtocolMessageType(
    "HelloResponse",
    (_message.Message,),
    dict(
        DESCRIPTOR=_HELLORESPONSE,
        __module__="api_pb2"
        # @@protoc_insertion_point(class_scope:HelloResponse)
    ),
)
_sym_db.RegisterMessage(HelloResponse)

ConnectRequest = _reflection.GeneratedProtocolMessageType(
    "ConnectRequest",
    (_message.Message,),
    dict(
        DESCRIPTOR=_CONNECTREQUEST,
        __module__="api_pb2"
        # @@protoc_insertion_point(class_scope:ConnectRequest)
    ),
)
_sym_db.RegisterMessage(ConnectRequest)

ConnectResponse = _reflection.GeneratedProtocolMessageType(
    "ConnectResponse",
    (_message.Message,),
    dict(
        DESCRIPTOR=_CONNECTRESPONSE,
        __module__="api_pb2"
        # @@protoc_insertion_point(class_scope:ConnectResponse)
    ),
)
_sym_db.RegisterMessage(ConnectResponse)

DisconnectRequest = _reflection.GeneratedProtocolMessageType(
    "DisconnectRequest",
    (_message.Message,),
    dict(
        DESCRIPTOR=_DISCONNECTREQUEST,
        __module__="api_pb2"
        # @@protoc_insertion_point(class_scope:DisconnectRequest)
    ),
)
_sym_db.RegisterMessage(DisconnectRequest)

DisconnectResponse = _reflection.GeneratedProtocolMessageType(
    "DisconnectResponse",
    (_message.Message,),
    dict(
        DESCRIPTOR=_DISCONNECTRESPONSE,
        __module__="api_pb2"
        # @@protoc_insertion_point(class_scope:DisconnectResponse)
    ),
)
_sym_db.RegisterMessage(DisconnectResponse)

PingRequest = _reflection.GeneratedProtocolMessageType(
    "PingRequest",
    (_message.Message,),
    dict(
        DESCRIPTOR=_PINGREQUEST,
        __module__="api_pb2"
        # @@protoc_insertion_point(class_scope:PingRequest)
    ),
)
_sym_db.RegisterMessage(PingRequest)

PingResponse = _reflection.GeneratedProtocolMessageType(
    "PingResponse",
    (_message.Message,),
    dict(
        DESCRIPTOR=_PINGRESPONSE,
        __module__="api_pb2"
        # @@protoc_insertion_point(class_scope:PingResponse)
    ),
)
_sym_db.RegisterMessage(PingResponse)

DeviceInfoRequest = _reflection.GeneratedProtocolMessageType(
    "DeviceInfoRequest",
    (_message.Message,),
    dict(
        DESCRIPTOR=_DEVICEINFOREQUEST,
        __module__="api_pb2"
        # @@protoc_insertion_point(class_scope:DeviceInfoRequest)
    ),
)
_sym_db.RegisterMessage(DeviceInfoRequest)

DeviceInfoResponse = _reflection.GeneratedProtocolMessageType(
    "DeviceInfoResponse",
    (_message.Message,),
    dict(
        DESCRIPTOR=_DEVICEINFORESPONSE,
        __module__="api_pb2"
        # @@protoc_insertion_point(class_scope:DeviceInfoResponse)
    ),
)
_sym_db.RegisterMessage(DeviceInfoResponse)

ListEntitiesRequest = _reflection.GeneratedProtocolMessageType(
    "ListEntitiesRequest",
    (_message.Message,),
    dict(
        DESCRIPTOR=_LISTENTITIESREQUEST,
        __module__="api_pb2"
        # @@protoc_insertion_point(class_scope:ListEntitiesRequest)
    ),
)
_sym_db.RegisterMessage(ListEntitiesRequest)

ListEntitiesBinarySensorResponse = _reflection.GeneratedProtocolMessageType(
    "ListEntitiesBinarySensorResponse",
    (_message.Message,),
    dict(
        DESCRIPTOR=_LISTENTITIESBINARYSENSORRESPONSE,
        __module__="api_pb2"
        # @@protoc_insertion_point(class_scope:ListEntitiesBinarySensorResponse)
    ),
)
_sym_db.RegisterMessage(ListEntitiesBinarySensorResponse)

ListEntitiesCoverResponse = _reflection.GeneratedProtocolMessageType(
    "ListEntitiesCoverResponse",
    (_message.Message,),
    dict(
        DESCRIPTOR=_LISTENTITIESCOVERRESPONSE,
        __module__="api_pb2"
        # @@protoc_insertion_point(class_scope:ListEntitiesCoverResponse)
    ),
)
_sym_db.RegisterMessage(ListEntitiesCoverResponse)

ListEntitiesFanResponse = _reflection.GeneratedProtocolMessageType(
    "ListEntitiesFanResponse",
    (_message.Message,),
    dict(
        DESCRIPTOR=_LISTENTITIESFANRESPONSE,
        __module__="api_pb2"
        # @@protoc_insertion_point(class_scope:ListEntitiesFanResponse)
    ),
)
_sym_db.RegisterMessage(ListEntitiesFanResponse)

ListEntitiesLightResponse = _reflection.GeneratedProtocolMessageType(
    "ListEntitiesLightResponse",
    (_message.Message,),
    dict(
        DESCRIPTOR=_LISTENTITIESLIGHTRESPONSE,
        __module__="api_pb2"
        # @@protoc_insertion_point(class_scope:ListEntitiesLightResponse)
    ),
)
_sym_db.RegisterMessage(ListEntitiesLightResponse)

ListEntitiesSensorResponse = _reflection.GeneratedProtocolMessageType(
    "ListEntitiesSensorResponse",
    (_message.Message,),
    dict(
        DESCRIPTOR=_LISTENTITIESSENSORRESPONSE,
        __module__="api_pb2"
        # @@protoc_insertion_point(class_scope:ListEntitiesSensorResponse)
    ),
)
_sym_db.RegisterMessage(ListEntitiesSensorResponse)

ListEntitiesSwitchResponse = _reflection.GeneratedProtocolMessageType(
    "ListEntitiesSwitchResponse",
    (_message.Message,),
    dict(
        DESCRIPTOR=_LISTENTITIESSWITCHRESPONSE,
        __module__="api_pb2"
        # @@protoc_insertion_point(class_scope:ListEntitiesSwitchResponse)
    ),
)
_sym_db.RegisterMessage(ListEntitiesSwitchResponse)

ListEntitiesTextSensorResponse = _reflection.GeneratedProtocolMessageType(
    "ListEntitiesTextSensorResponse",
    (_message.Message,),
    dict(
        DESCRIPTOR=_LISTENTITIESTEXTSENSORRESPONSE,
        __module__="api_pb2"
        # @@protoc_insertion_point(class_scope:ListEntitiesTextSensorResponse)
    ),
)
_sym_db.RegisterMessage(ListEntitiesTextSensorResponse)

ListEntitiesDoneResponse = _reflection.GeneratedProtocolMessageType(
    "ListEntitiesDoneResponse",
    (_message.Message,),
    dict(
        DESCRIPTOR=_LISTENTITIESDONERESPONSE,
        __module__="api_pb2"
        # @@protoc_insertion_point(class_scope:ListEntitiesDoneResponse)
    ),
)
_sym_db.RegisterMessage(ListEntitiesDoneResponse)

SubscribeStatesRequest = _reflection.GeneratedProtocolMessageType(
    "SubscribeStatesRequest",
    (_message.Message,),
    dict(
        DESCRIPTOR=_SUBSCRIBESTATESREQUEST,
        __module__="api_pb2"
        # @@protoc_insertion_point(class_scope:SubscribeStatesRequest)
    ),
)
_sym_db.RegisterMessage(SubscribeStatesRequest)

BinarySensorStateResponse = _reflection.GeneratedProtocolMessageType(
    "BinarySensorStateResponse",
    (_message.Message,),
    dict(
        DESCRIPTOR=_BINARYSENSORSTATERESPONSE,
        __module__="api_pb2"
        # @@protoc_insertion_point(class_scope:BinarySensorStateResponse)
    ),
)
_sym_db.RegisterMessage(BinarySensorStateResponse)

CoverStateResponse = _reflection.GeneratedProtocolMessageType(
    "CoverStateResponse",
    (_message.Message,),
    dict(
        DESCRIPTOR=_COVERSTATERESPONSE,
        __module__="api_pb2"
        # @@protoc_insertion_point(class_scope:CoverStateResponse)
    ),
)
_sym_db.RegisterMessage(CoverStateResponse)

FanStateResponse = _reflection.GeneratedProtocolMessageType(
    "FanStateResponse",
    (_message.Message,),
    dict(
        DESCRIPTOR=_FANSTATERESPONSE,
        __module__="api_pb2"
        # @@protoc_insertion_point(class_scope:FanStateResponse)
    ),
)
_sym_db.RegisterMessage(FanStateResponse)

LightStateResponse = _reflection.GeneratedProtocolMessageType(
    "LightStateResponse",
    (_message.Message,),
    dict(
        DESCRIPTOR=_LIGHTSTATERESPONSE,
        __module__="api_pb2"
        # @@protoc_insertion_point(class_scope:LightStateResponse)
    ),
)
_sym_db.RegisterMessage(LightStateResponse)

SensorStateResponse = _reflection.GeneratedProtocolMessageType(
    "SensorStateResponse",
    (_message.Message,),
    dict(
        DESCRIPTOR=_SENSORSTATERESPONSE,
        __module__="api_pb2"
        # @@protoc_insertion_point(class_scope:SensorStateResponse)
    ),
)
_sym_db.RegisterMessage(SensorStateResponse)

SwitchStateResponse = _reflection.GeneratedProtocolMessageType(
    "SwitchStateResponse",
    (_message.Message,),
    dict(
        DESCRIPTOR=_SWITCHSTATERESPONSE,
        __module__="api_pb2"
        # @@protoc_insertion_point(class_scope:SwitchStateResponse)
    ),
)
_sym_db.RegisterMessage(SwitchStateResponse)

TextSensorStateResponse = _reflection.GeneratedProtocolMessageType(
    "TextSensorStateResponse",
    (_message.Message,),
    dict(
        DESCRIPTOR=_TEXTSENSORSTATERESPONSE,
        __module__="api_pb2"
        # @@protoc_insertion_point(class_scope:TextSensorStateResponse)
    ),
)
_sym_db.RegisterMessage(TextSensorStateResponse)

CoverCommandRequest = _reflection.GeneratedProtocolMessageType(
    "CoverCommandRequest",
    (_message.Message,),
    dict(
        DESCRIPTOR=_COVERCOMMANDREQUEST,
        __module__="api_pb2"
        # @@protoc_insertion_point(class_scope:CoverCommandRequest)
    ),
)
_sym_db.RegisterMessage(CoverCommandRequest)

FanCommandRequest = _reflection.GeneratedProtocolMessageType(
    "FanCommandRequest",
    (_message.Message,),
    dict(
        DESCRIPTOR=_FANCOMMANDREQUEST,
        __module__="api_pb2"
        # @@protoc_insertion_point(class_scope:FanCommandRequest)
    ),
)
_sym_db.RegisterMessage(FanCommandRequest)

LightCommandRequest = _reflection.GeneratedProtocolMessageType(
    "LightCommandRequest",
    (_message.Message,),
    dict(
        DESCRIPTOR=_LIGHTCOMMANDREQUEST,
        __module__="api_pb2"
        # @@protoc_insertion_point(class_scope:LightCommandRequest)
    ),
)
_sym_db.RegisterMessage(LightCommandRequest)

SwitchCommandRequest = _reflection.GeneratedProtocolMessageType(
    "SwitchCommandRequest",
    (_message.Message,),
    dict(
        DESCRIPTOR=_SWITCHCOMMANDREQUEST,
        __module__="api_pb2"
        # @@protoc_insertion_point(class_scope:SwitchCommandRequest)
    ),
)
_sym_db.RegisterMessage(SwitchCommandRequest)

SubscribeLogsRequest = _reflection.GeneratedProtocolMessageType(
    "SubscribeLogsRequest",
    (_message.Message,),
    dict(
        DESCRIPTOR=_SUBSCRIBELOGSREQUEST,
        __module__="api_pb2"
        # @@protoc_insertion_point(class_scope:SubscribeLogsRequest)
    ),
)
_sym_db.RegisterMessage(SubscribeLogsRequest)

SubscribeLogsResponse = _reflection.GeneratedProtocolMessageType(
    "SubscribeLogsResponse",
    (_message.Message,),
    dict(
        DESCRIPTOR=_SUBSCRIBELOGSRESPONSE,
        __module__="api_pb2"
        # @@protoc_insertion_point(class_scope:SubscribeLogsResponse)
    ),
)
_sym_db.RegisterMessage(SubscribeLogsResponse)

SubscribeServiceCallsRequest = _reflection.GeneratedProtocolMessageType(
    "SubscribeServiceCallsRequest",
    (_message.Message,),
    dict(
        DESCRIPTOR=_SUBSCRIBESERVICECALLSREQUEST,
        __module__="api_pb2"
        # @@protoc_insertion_point(class_scope:SubscribeServiceCallsRequest)
    ),
)
_sym_db.RegisterMessage(SubscribeServiceCallsRequest)

ServiceCallResponse = _reflection.GeneratedProtocolMessageType(
    "ServiceCallResponse",
    (_message.Message,),
    dict(
        DataEntry=_reflection.GeneratedProtocolMessageType(
            "DataEntry",
            (_message.Message,),
            dict(
                DESCRIPTOR=_SERVICECALLRESPONSE_DATAENTRY,
                __module__="api_pb2"
                # @@protoc_insertion_point(class_scope:ServiceCallResponse.DataEntry)
            ),
        ),
        DataTemplateEntry=_reflection.GeneratedProtocolMessageType(
            "DataTemplateEntry",
            (_message.Message,),
            dict(
                DESCRIPTOR=_SERVICECALLRESPONSE_DATATEMPLATEENTRY,
                __module__="api_pb2"
                # @@protoc_insertion_point(class_scope:ServiceCallResponse.DataTemplateEntry)
            ),
        ),
        VariablesEntry=_reflection.GeneratedProtocolMessageType(
            "VariablesEntry",
            (_message.Message,),
            dict(
                DESCRIPTOR=_SERVICECALLRESPONSE_VARIABLESENTRY,
                __module__="api_pb2"
                # @@protoc_insertion_point(class_scope:ServiceCallResponse.VariablesEntry)
            ),
        ),
        DESCRIPTOR=_SERVICECALLRESPONSE,
        __module__="api_pb2"
        # @@protoc_insertion_point(class_scope:ServiceCallResponse)
    ),
)
_sym_db.RegisterMessage(ServiceCallResponse)
_sym_db.RegisterMessage(ServiceCallResponse.DataEntry)
_sym_db.RegisterMessage(ServiceCallResponse.DataTemplateEntry)
_sym_db.RegisterMessage(ServiceCallResponse.VariablesEntry)

SubscribeHomeAssistantStatesRequest = _reflection.GeneratedProtocolMessageType(
    "SubscribeHomeAssistantStatesRequest",
    (_message.Message,),
    dict(
        DESCRIPTOR=_SUBSCRIBEHOMEASSISTANTSTATESREQUEST,
        __module__="api_pb2"
        # @@protoc_insertion_point(class_scope:SubscribeHomeAssistantStatesRequest)
    ),
)
_sym_db.RegisterMessage(SubscribeHomeAssistantStatesRequest)

SubscribeHomeAssistantStateResponse = _reflection.GeneratedProtocolMessageType(
    "SubscribeHomeAssistantStateResponse",
    (_message.Message,),
    dict(
        DESCRIPTOR=_SUBSCRIBEHOMEASSISTANTSTATERESPONSE,
        __module__="api_pb2"
        # @@protoc_insertion_point(class_scope:SubscribeHomeAssistantStateResponse)
    ),
)
_sym_db.RegisterMessage(SubscribeHomeAssistantStateResponse)

HomeAssistantStateResponse = _reflection.GeneratedProtocolMessageType(
    "HomeAssistantStateResponse",
    (_message.Message,),
    dict(
        DESCRIPTOR=_HOMEASSISTANTSTATERESPONSE,
        __module__="api_pb2"
        # @@protoc_insertion_point(class_scope:HomeAssistantStateResponse)
    ),
)
_sym_db.RegisterMessage(HomeAssistantStateResponse)

GetTimeRequest = _reflection.GeneratedProtocolMessageType(
    "GetTimeRequest",
    (_message.Message,),
    dict(
        DESCRIPTOR=_GETTIMEREQUEST,
        __module__="api_pb2"
        # @@protoc_insertion_point(class_scope:GetTimeRequest)
    ),
)
_sym_db.RegisterMessage(GetTimeRequest)

GetTimeResponse = _reflection.GeneratedProtocolMessageType(
    "GetTimeResponse",
    (_message.Message,),
    dict(
        DESCRIPTOR=_GETTIMERESPONSE,
        __module__="api_pb2"
        # @@protoc_insertion_point(class_scope:GetTimeResponse)
    ),
)
_sym_db.RegisterMessage(GetTimeResponse)


_SERVICECALLRESPONSE_DATAENTRY._options = None
_SERVICECALLRESPONSE_DATATEMPLATEENTRY._options = None
_SERVICECALLRESPONSE_VARIABLESENTRY._options = None
# @@protoc_insertion_point(module_scope)
